import numpy as np
import pylab as plt
import pandas as pd
from mpl_toolkits.basemap import Basemap
import matplotlib
import matplotlib.image as image
import netCDF4
import matplotlib.mlab as mlab

plt.rcParams['text.usetex'] = False


# Initialisation of colormaps 
cmappie = plt.cm.hsv  # pie colormap
cmaporca = plt.cm.viridis_r # pcolor colormap
piealpha = 1 # transparency for pie plots

# initialisation of domain boundaries
# from which to extract the data
lonmin, lonmax = 0, 360
latmin, latmax = -40, 40
function_name = 'sqrt' # name of the function for radius scaling ('power' or 'sqrt')

if function_name == 'power':
    fun_to_apply = lambda x: x**2
else:
    fun_to_apply = lambda x: np.sqrt(x)

# initialisation of map projection (mollweide)
mmap = Basemap(projection='moll', 
        lon_0 = 0.5*(lonmin+lonmax))

# load APECOSM data from netcdf file
dirin = '../data/'
fin = netCDF4.Dataset(dirin + 'OOPE_spinup_Y50D009_small.nc', 'r')
var = fin.variables['OOPE'][0, :, :, 0, 0]
lonmod = fin.variables['longitude'][:]
latmod = fin.variables['latitude'][:]
fin.close()

# Interpolation of Apecosm onto regular grid centerred on pacific
pas = 1  # 1 degree resolution 
lonmodnew=np.arange(0, 360+pas, pas)  # interpolated long.
latmodnew=np.arange(-90, 90+pas, pas)  # interp. latitude
lon1d = np.ravel(lonmod)  # conversion of model lon. into 1D array
lon1d[lon1d<0] += 360  # shift the model grid (centerring in Pacific)
varmodnew = mlab.griddata(lon1d, np.ravel(latmod), np.ravel(var), lonmodnew, latmodnew, interp='linear')  # interpolation
varmodnew = np.ma.masked_where(varmodnew!=varmodnew, varmodnew)  # masking data

lonmodnew, latmodnew = np.meshgrid(lonmodnew, latmodnew)  # conversion of 1D interp. lon/lat into 2D arrays
xmodnew, ymodnew = mmap(lonmodnew, latmodnew)  # conversion of lon/lat into X/Y

# Initialisation of figure
fig = plt.figure()
ax = plt.gca()
plt.subplots_adjust(left=0.05, right=0.95, top=0.95) # handling the map position 
plt.title('My title')  # adding title
mmap.drawcoastlines(linewidth=0.5)  # adding coastlines
mmap.fillcontinents(color='lightgray') # adding rivers
mmap.drawcountries(color='k') # adding countries
cs = mmap.pcolormesh(xmodnew, ymodnew, varmodnew, cmap=cmaporca) # pcolormesh of model data
cb = mmap.colorbar(cs, location='bottom', pad='2%') # adding the colorbar
cb.set_label('My legend', fontsize=10)  # set the colorbar title 
plt.setp(cb.ax.get_xticklabels(), fontsize=10) # change the labels on colorbar. for vertical cbar, use "yticklabels"

# change the size of the offset string ("e11")
t = cb.ax.xaxis.get_offset_text()  
t.set_size(10)

# reading the sardara dataset
data = pd.read_csv('../data/sardara_yft_55_1950_2014_reshape.csv', index_col=0)
data.loc[data.long_centroid<=0, 'long_centroid'] += 360 # shift grid

data = data[(data.long_centroid <= lonmax) & (data.long_centroid >= lonmin)] # extract data according to lon
data = data[(data.lat_centroid <= latmax) & (data.lat_centroid >= latmin)] # extract data according to lat

nlines = len(data.index) # count the number of lines

discard = ['long_centroid', 'lat_centroid', 'TOTAL'] # data labels to discard
varlist = [d for d in data.columns if d not in discard]  # list of "data" to draw in pie charts
nvar = len(varlist) # number of variables in pie charts

# extract the list of RGB arrays according to colormap and variable number
colors = [cmappie(float(p)/nvar) for p in range(0, nvar)] 

piesettings = {'shadow':True, 'startangle':90, 'frame':True, 'colors':colors, 'wedgeprops':{'linewidth':0.5}}

prop = data.loc[:,varlist].values # extract the proportion array
lon = data.long_centroid.values # extract the lon array
lat = data.lat_centroid.values # extract the lat array
total = data.TOTAL.values # extract the total variable

prop /= np.tile(total, (nvar, 1)).T # computes the proportion by dividing by the total
prop[np.isnan(prop)] = 0 # set filled values to 0

# draw map backgrounds (lon, lat
#mmap.fillcontinents('lightgray') 
pas = 40; lonticks = np.arange(lonmin, lonmax+pas, pas) # sets the lon ticks locations
pas = 20; latticks = np.arange(latmin, latmax+pas, pas) # sets the lat ticks locations
mmap.drawparallels(latticks, labels=[1,0,0,0], fontsize=8)
mmap.drawmeridians(lonticks, labels=[0,0,0,1], fontsize=8)

# computes the minimum and maximum radius values as a function of map deltax/deltay
# converts lon/lat 1D arrays into a 2D grid of array
lon2d = np.unique(lon)
lat2d = np.unique(lat)
lon2d, lat2d = np.meshgrid(lon2d, lat2d)
# transforming from lat/lon into map X/Y
x2d, y2d = mmap(lon2d, lat2d)
dx = np.diff(x2d, axis=1).min()  # computes the minimum dx distance 
dy = np.diff(y2d, axis=0).min()# computes the minimum dx distance 
radmax = np.min([dx, dy]) # computes the min of dx/dy
radmin = radmax / 100. # sets radmin as radmax.100

# defines the corrected radius depending on radmin, radmax, and total values
min_catch, max_catch = total.min(), total.max()
a = (radmin-radmax)/(fun_to_apply(min_catch)-fun_to_apply(max_catch))
b = radmax-a*fun_to_apply(max_catch)
radcorr = a*fun_to_apply(total)+b

# loop over all the elements of the lon/lat/radcorr/prop lists/arrays
for lontemp, lattemp, rtemp, proptemp in  zip(lon, lat, radcorr, prop)[:]:

    # convers pie lon/lat into map coords
    x, y = mmap(lontemp, lattemp)

    pieplot = plt.pie(proptemp, radius=rtemp, center=(x, y), **piesettings)
    [temp.set_alpha(piealpha) for temp in pieplot[0]] # set transparency

# add the pie chart legend
legx = mmap.llcrnrx - (mmap.llcrnry - mmap.llcrnrx)*0.5
prop = matplotlib.font_manager.FontProperties(size=7)
plt.legend(pieplot[0], varlist, bbox_to_anchor=(0, -0.25, 0.5, 0.1), loc=2,
                   ncol=10, mode=None, borderaxespad=0., prop=prop)

# sets the x/y coordinates for the 
# legend (leg=pie location, text=pie label location)
xleg = (mmap.llcrnrx) + 0.9*(mmap.urcrnrx - mmap.llcrnrx)
yleg = (mmap.llcrnry) + 12/100.*(mmap.urcrnry - mmap.llcrnry)
ytext = (mmap.llcrnry) + 5/100.*(mmap.urcrnry - mmap.llcrnry)

# add the legend for pie size
# sets a rounded total value
N = 100
total = np.max(total).astype(np.int)
total = int(np.ceil(total/N)*N)

# defines the pie to plot in the legend
proptemp = [1./nvar] * nvar 
toto = plt.pie(proptemp, radius=a*fun_to_apply(total)+b, center=(xleg, yleg), **piesettings)
[temp.set_alpha(piealpha) for temp in toto[0]] # set transparency
# add the pie label
plt.text(xleg, ytext, '%d tons' %total, ha='center', va='center', fontsize=8)

# add the logo on a new map
im = image.imread('../data/logo-ird.jpg')
newax = fig.add_axes([0.12, 0.85, 0.15, 0.1], anchor='NE', zorder=-1)
newax.imshow(im, aspect='equal')
newax.axis('off')

# save the figure
plt.savefig('sardara.png', bbox_inches='tight')

